@@ -437,7 +437,6 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_CONSTANT_ENCAPSED_STRING:
             {
                 // support something like `include dirname(__FILE__) . "/...`
-                ///TODO: include __DIR__ . "/ (php 5.3)
                 bool isAfterDirname = false;
                 //NOTE: prependedBy will return -1 on failure, this is what we need in these cases
                 //      on success it will return a positive number, we'll need to switch it's sign in that case
@@ -449,7 +448,16 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
                     if ( lastToken.stringAt(relPos + 1).compare(QLatin1String("dirname"), Qt::CaseInsensitive) == 0 ) {
                         isAfterDirname = true;
                     }
+                } else {
+                    relPos = lastToken.prependedBy(TokenList() << Parser::Token_CONCAT << Parser::Token_DIR, true);
+
+                    if ( relPos != -1 ) {
+                        // switch sign
+                        relPos = -relPos;
+                        isAfterDirname = true;
+                    }
                 }
+
                 skipWhiteSpace(lastToken, relPos);
                 if ( lastToken.typeAt(relPos) == Parser::Token_LPAREN ) {
                     --relPos;
@@ -600,7 +608,6 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_ARRAY:
         case Parser::Token_AS:
         case Parser::Token_BACKTICK:
-        case Parser::Token_BOOL:
         case Parser::Token_BREAK:
         case Parser::Token_CALLABLE:
         case Parser::Token_CASE:
@@ -612,6 +619,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_CONTINUE:
         case Parser::Token_DECLARE:
         case Parser::Token_DEFAULT:
+        case Parser::Token_DIR:
         case Parser::Token_DNUMBER:
         case Parser::Token_DO:
         case Parser::Token_DOLLAR:
@@ -630,7 +638,6 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_EVAL:
         case Parser::Token_FILE:
         case Parser::Token_FINALLY:
-        case Parser::Token_FLOAT:
         case Parser::Token_FOR:
         case Parser::Token_FOREACH:
         case Parser::Token_FUNCTION:
@@ -642,11 +649,9 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_INCLUDE_ONCE:
         case Parser::Token_INLINE_HTML:
         case Parser::Token_INSTEADOF:
-        case Parser::Token_INT:
         case Parser::Token_INTERFACE:
         case Parser::Token_INVALID:
         case Parser::Token_ISSET:
-        case Parser::Token_ITERABLE:
         case Parser::Token_LINE:
         case Parser::Token_LIST:
         case Parser::Token_LNUMBER:
@@ -657,10 +662,10 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_REQUIRE_ONCE:
         case Parser::Token_RBRACKET:
         case Parser::Token_RPAREN:
-        case Parser::Token_STRING_TYPE:
         case Parser::Token_STRING_VARNAME:
         case Parser::Token_SWITCH:
         case Parser::Token_TRAIT:
+        case Parser::Token_TRAIT_C:
         case Parser::Token_TRY:
         case Parser::Token_UNSET:
         case Parser::Token_USE:
@@ -668,6 +673,8 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_VOID:
         case Parser::Token_WHILE:
         case Parser::Token_WHITESPACE:
+        case Parser::Token_YIELD:
+        case Parser::Token_YIELD_FROM:
         /// TODO: code completion after goto
         case Parser::Token_GOTO:
         case Parser::TokenTypeSize:
@@ -1276,11 +1283,11 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                         if (member) {
                             if (decl.second == 0) {
                                 // this function is already implemented
-                                alreadyImplemented << decl.first->indexedIdentifier().getIndex();
+                                alreadyImplemented << decl.first->indexedIdentifier().index();
                                 continue;
                             }
                             // skip already implemented functions
-                            if (alreadyImplemented.contains(decl.first->indexedIdentifier().getIndex())) {
+                            if (alreadyImplemented.contains(decl.first->indexedIdentifier().index())) {
                                 continue;
                             }
                             // skip non-static functions when requested
@@ -1298,13 +1305,13 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                             // skip public functions when requested
                             if (filterPublic && member->accessPolicy() == Declaration::Public) {
                                 // make sure no non-public base members are added
-                                alreadyImplemented << decl.first->indexedIdentifier().getIndex();
+                                alreadyImplemented << decl.first->indexedIdentifier().index();
                                 continue;
                             }
                             // skip final members
                             if (classFunc && classFunc->isFinal()) {
                                 // make sure no non-final base members are added
-                                alreadyImplemented << decl.first->indexedIdentifier().getIndex();
+                                alreadyImplemented << decl.first->indexedIdentifier().index();
                                 continue;
                             }
                             // make sure we inherit or implement the base class of this member
@@ -1330,7 +1337,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                             items << CompletionTreeItemPointer(new ImplementationItem(itype, DeclarationPointer(decl.first),
                                                                 Php::CodeCompletionContext::Ptr(this), decl.second));
                             // don't add identical items twice to the completion choices
-                            alreadyImplemented << decl.first->indexedIdentifier().getIndex();
+                            alreadyImplemented << decl.first->indexedIdentifier().index();
                         }
                     }
                 }
@@ -1443,14 +1450,14 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
     } else {
         //Show all visible declarations
         QSet<uint> existingIdentifiers;
-        QList<DeclarationDepthPair> decls = m_duContext->allDeclarations(
+        const auto decls = m_duContext->allDeclarations(
             CursorInRevision::invalid(),
             m_duContext->topContext()
         );
 
         qCDebug(COMPLETION) << "setContext: using all declarations visible:" << decls.size();
 
-        QListIterator<DeclarationDepthPair> i(decls);
+        QVectorIterator<DeclarationDepthPair> i(decls);
         i.toBack();
         while (i.hasPrevious()) {
             DeclarationDepthPair decl = i.previous();
@@ -1464,8 +1471,8 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                     }
                 }
 
-                if (existingIdentifiers.contains(dec->indexedIdentifier().getIndex())) continue;
-                existingIdentifiers.insert(dec->indexedIdentifier().getIndex());
+                if (existingIdentifiers.contains(dec->indexedIdentifier().index())) continue;
+                existingIdentifiers.insert(dec->indexedIdentifier().index());
             }
             if (abort)
                 return items;
@@ -1554,7 +1561,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
             if (parentContext->memberAccessOperation() == CodeCompletionContext::FunctionCallAccess) {
                 if (!parentContext->memberAccessContainer().allDeclarationIds().isEmpty()) {
                     Declaration* decl = parentContext->memberAccessContainer().allDeclarationIds().first()
-                                            .getDeclaration(m_duContext->topContext());
+                                            .declaration(m_duContext->topContext());
 
                     if (!isValidCompletionItem(decl)) {
                         continue;
